package sun.management;

import java.util.Comparator;
import java.lang.reflect.InvocationTargetException;
import java.security.Permission;
import javax.management.ListenerNotFoundException;
import javax.management.NotificationFilter;
import javax.management.NotificationListener;
import javax.management.Notification;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;
import java.util.Iterator;
import javax.management.Descriptor;
import javax.management.ImmutableDescriptor;
import java.util.Collections;
import javax.management.MBeanOperationInfo;
import java.util.HashMap;
import javax.management.MBeanParameterInfo;
import java.util.TreeSet;
import javax.management.MBeanInfo;
import javax.management.AttributeList;
import javax.management.InvalidAttributeValueException;
import javax.management.Attribute;
import javax.management.ReflectionException;
import javax.management.MBeanException;
import javax.management.AttributeNotFoundException;
import javax.management.MBeanNotificationInfo;
import java.util.Map;
import com.sun.management.DiagnosticCommandMBean;

class DiagnosticCommandImpl extends NotificationEmitterSupport implements DiagnosticCommandMBean
{
    private final VMManagement jvm;
    private volatile Map<String, DiagnosticCommandImpl.Wrapper> wrappers;
    private static final String strClassName;
    private static final String strArrayClassName;
    private final boolean isSupported;
    private static final String notifName = "javax.management.Notification";
    private static final String[] diagFramNotifTypes;
    private MBeanNotificationInfo[] notifInfo;
    private static long seqNumber;
    
    @Override
    public Object getAttribute(String message) throws AttributeNotFoundException, MBeanException, ReflectionException {
        throw new AttributeNotFoundException(message);
    }
    
    @Override
    public void setAttribute(Attribute attribute) throws AttributeNotFoundException, InvalidAttributeValueException, MBeanException, ReflectionException {
        throw new AttributeNotFoundException(attribute.getName());
    }
    
    @Override
    public AttributeList getAttributes(String[] array) {
        return new AttributeList();
    }
    
    @Override
    public AttributeList setAttributes(AttributeList list) {
        return new AttributeList();
    }
    
    DiagnosticCommandImpl(VMManagement jvm) {
        this.wrappers = null;
        this.notifInfo = null;
        this.jvm = jvm;
        this.isSupported = jvm.isRemoteDiagnosticCommandsSupported();
    }
    
    @Override
    public MBeanInfo getMBeanInfo() {
        TreeSet<MBeanOperationInfo> set = new TreeSet(new DiagnosticCommandImpl.OperationInfoComparator());
        Map<String, Object> m;
        if (!this.isSupported) {
            m = Collections.EMPTY_MAP;
        }
        else {
            try {
                String[] diagnosticCommands = this.getDiagnosticCommands();
                DiagnosticCommandInfo[] diagnosticCommandInfo = this.getDiagnosticCommandInfo(diagnosticCommands);
                MBeanParameterInfo[] array = { new MBeanParameterInfo("arguments", DiagnosticCommandImpl.strArrayClassName, "Array of Diagnostic Commands Arguments and Options") };
                m = new HashMap();
                for (int i = 0; i < diagnosticCommands.length; ++i) {
                    String transform = transform(diagnosticCommands[i]);
                    try {
                        DiagnosticCommandImpl.Wrapper o = new DiagnosticCommandImpl.Wrapper(transform, diagnosticCommands[i], diagnosticCommandInfo[i]);
                        m.put(transform, o);
                        set.add(new MBeanOperationInfo(o.name, o.info.getDescription(), (o.info.getArgumentsInfo() == null || o.info.getArgumentsInfo().isEmpty()) ? null : array, DiagnosticCommandImpl.strClassName, 2, this.commandDescriptor(o)));
                    }
                    catch (InstantiationException ex) {}
                }
            }
            catch (IllegalArgumentException | UnsupportedOperationException ex2) {
                m = Collections.EMPTY_MAP;
            }
        }
        this.wrappers = Collections.unmodifiableMap(m);
        HashMap fields = new HashMap();
        fields.put("immutableInfo", "false");
        fields.put("interfaceClassName", "com.sun.management.DiagnosticCommandMBean");
        fields.put("mxbean", "false");
        return new MBeanInfo(this.getClass().getName(), "Diagnostic Commands", null, null, (MBeanOperationInfo[])set.toArray(new MBeanOperationInfo[set.size()]), this.getNotificationInfo(), new ImmutableDescriptor(fields));
    }
    
    @Override
    public Object invoke(String s, Object[] array, String[] array2) throws MBeanException, ReflectionException {
        if (!this.isSupported) {
            throw new UnsupportedOperationException();
        }
        if (this.wrappers == null) {
            this.getMBeanInfo();
        }
        DiagnosticCommandImpl.Wrapper wrapper = (DiagnosticCommandImpl.Wrapper)this.wrappers.get(s);
        if (wrapper != null) {
            if (wrapper.info.getArgumentsInfo().isEmpty() && (array == null || array.length == 0) && (array2 == null || array2.length == 0)) {
                return wrapper.execute((String[])null);
            }
            if (array != null && array.length == 1 && array2 != null && array2.length == 1 && array2[0] != null && array2[0].compareTo(DiagnosticCommandImpl.strArrayClassName) == 0) {
                return wrapper.execute((String[])(String[])array[0]);
            }
        }
        throw new ReflectionException(new NoSuchMethodException(s));
    }
    
    private static String transform(String s) {
        StringBuilder sb = new StringBuilder();
        boolean n = true;
        int n2 = 0;
        for (int i = 0; i < s.length(); ++i) {
            int char1 = s.charAt(i);
            if (char1 == '.' || char1 == '_') {
                n = false;
                n2 = 1;
            }
            else if (n2 != 0) {
                n2 = 0;
                sb.append(Character.toUpperCase((char)char1));
            }
            else if (!(n == 0)) {
                sb.append(Character.toLowerCase((char)char1));
            }
            else {
                sb.append((char)char1);
            }
        }
        return sb.toString();
    }
    
    private Descriptor commandDescriptor(DiagnosticCommandImpl.Wrapper wrapper) throws IllegalArgumentException {
        HashMap<String, Object> fields = new HashMap();
        fields.put("dcmd.name", wrapper.info.getName());
        fields.put("dcmd.description", wrapper.info.getDescription());
        fields.put("dcmd.vmImpact", wrapper.info.getImpact());
        fields.put("dcmd.permissionClass", wrapper.info.getPermissionClass());
        fields.put("dcmd.permissionName", wrapper.info.getPermissionName());
        fields.put("dcmd.permissionAction", wrapper.info.getPermissionAction());
        fields.put("dcmd.enabled", Boolean.valueOf(wrapper.info.isEnabled()));
        StringBuilder sb = new StringBuilder();
        sb.append("help ");
        sb.append(wrapper.info.getName());
        fields.put("dcmd.help", this.executeDiagnosticCommand(sb.toString()));
        if (wrapper.info.getArgumentsInfo() != null && !wrapper.info.getArgumentsInfo().isEmpty()) {
            HashMap<String, ImmutableDescriptor> fields2 = new HashMap();
            for (DiagnosticCommandArgumentInfo diagnosticCommandArgumentInfo : wrapper.info.getArgumentsInfo()) {
                HashMap fields3 = new HashMap();
                fields3.put("dcmd.arg.name", diagnosticCommandArgumentInfo.getName());
                fields3.put("dcmd.arg.type", diagnosticCommandArgumentInfo.getType());
                fields3.put("dcmd.arg.description", diagnosticCommandArgumentInfo.getDescription());
                fields3.put("dcmd.arg.isMandatory", Boolean.valueOf(diagnosticCommandArgumentInfo.isMandatory()));
                fields3.put("dcmd.arg.isMultiple", Boolean.valueOf(diagnosticCommandArgumentInfo.isMultiple()));
                int option = diagnosticCommandArgumentInfo.isOption();
                fields3.put("dcmd.arg.isOption", Boolean.valueOf((boolean)option));
                if (option == 0) {
                    fields3.put("dcmd.arg.position", Integer.valueOf(diagnosticCommandArgumentInfo.getPosition()));
                }
                else {
                    fields3.put("dcmd.arg.position", Integer.valueOf(-1));
                }
                fields2.put(diagnosticCommandArgumentInfo.getName(), new ImmutableDescriptor(fields3));
            }
            fields.put("dcmd.arguments", new ImmutableDescriptor(fields2));
        }
        return new ImmutableDescriptor(fields);
    }
    
    @Override
    public MBeanNotificationInfo[] getNotificationInfo() {
        synchronized (this) {
            if (this.notifInfo == null) {
                (this.notifInfo = new MBeanNotificationInfo[1])[0] = new MBeanNotificationInfo(DiagnosticCommandImpl.diagFramNotifTypes, "javax.management.Notification", "Diagnostic Framework Notification");
            }
        }
        return (MBeanNotificationInfo[])this.notifInfo.clone();
    }
    
    private static long getNextSeqNumber() {
        return ++DiagnosticCommandImpl.seqNumber;
    }
    
    private void createDiagnosticFrameworkNotification() {
        if (!this.hasListeners()) {
            return;
        }
        ObjectName instance = null;
        try {
            instance = ObjectName.getInstance("com.sun.management:type=DiagnosticCommand");
        }
        catch (MalformedObjectNameException ex) {}
        Notification o = new Notification("jmx.mbean.info.changed", instance, getNextSeqNumber());
        o.setUserData(this.getMBeanInfo());
        this.sendNotification(o);
        return;
    }
    
    @Override
    public synchronized void addNotificationListener(NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) {
        int hasListeners = this.hasListeners();
        super.addNotificationListener(notificationListener, notificationFilter, o);
        int hasListeners2 = this.hasListeners();
        if (hasListeners == 0 && hasListeners2) {
            this.setNotificationEnabled(true);
        }
    }
    
    @Override
    public synchronized void removeNotificationListener(NotificationListener notificationListener) throws ListenerNotFoundException {
        int hasListeners = this.hasListeners();
        super.removeNotificationListener(notificationListener);
        int hasListeners2 = this.hasListeners();
        if (hasListeners && hasListeners2 == 0) {
            this.setNotificationEnabled(false);
        }
    }
    
    @Override
    public synchronized void removeNotificationListener(NotificationListener notificationListener, NotificationFilter notificationFilter, Object o) throws ListenerNotFoundException {
        int hasListeners = this.hasListeners();
        super.removeNotificationListener(notificationListener, notificationFilter, o);
        int hasListeners2 = this.hasListeners();
        if (hasListeners && hasListeners2 == 0) {
            this.setNotificationEnabled(false);
        }
    }
    
    private native void setNotificationEnabled(boolean p0);
    
    private native String[] getDiagnosticCommands();
    
    private native DiagnosticCommandInfo[] getDiagnosticCommandInfo(String[] p0);
    
    private native String executeDiagnosticCommand(String p0);
    
    static String access$000(DiagnosticCommandImpl diagnosticCommandImpl, String s) {
        return diagnosticCommandImpl.executeDiagnosticCommand(s);
    }
    
    static {
        strClassName = "".getClass().getName();
        strArrayClassName = String[].class.getName();
        diagFramNotifTypes = new String[] { "jmx.mbean.info.changed" };
        DiagnosticCommandImpl.seqNumber = 0L;
    }
    
    private class Wrapper
    {
        String name;
        String cmd;
        DiagnosticCommandInfo info;
        Permission permission;
        
        Wrapper(String name, String cmd, DiagnosticCommandInfo info) throws InstantiationException {
            this.name = name;
            this.cmd = cmd;
            this.info = info;
            this.permission = null;
            InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException | NoSuchMethodException | SecurityException cause = null;
            if (info.getPermissionClass() != null) {
                try {
                    Class<Object> forName = Class.forName(info.getPermissionClass());
                    if (info.getPermissionAction() == null) {
                        try {
                            this.permission = (Permission)forName.getConstructor(new Class[] { String.class }).newInstance(new Object[] { info.getPermissionName() });
                        }
                        catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException | NoSuchMethodException | SecurityException ex) {
                            cause = ex;
                        }
                    }
                    if (this.permission == null) {
                        try {
                            this.permission = (Permission)forName.getConstructor(new Class[] { String.class, String.class }).newInstance(new Object[] { info.getPermissionName(), info.getPermissionAction() });
                        }
                        catch (InstantiationException | IllegalAccessException | IllegalArgumentException | InvocationTargetException | NoSuchMethodException | SecurityException ex2) {
                            cause = ex2;
                        }
                    }
                }
                catch (ClassNotFoundException ex3) {}
                if (this.permission == null) {
                    new InstantiationException("Unable to instantiate required permission").initCause(cause);
                }
            }
        }
        
        public String execute(String[] array) {
            if (this.permission != null) {
                SecurityManager securityManager = System.getSecurityManager();
                if (securityManager != null) {
                    securityManager.checkPermission(this.permission);
                }
            }
            if (array == null) {
                return DiagnosticCommandImpl.access$000(DiagnosticCommandImpl.this, this.cmd);
            }
            StringBuilder o = new StringBuilder();
            o.append(this.cmd);
            int i = 0;
            while (i < array.length) {
                if (array[i] == null) {
                    throw new IllegalArgumentException("Invalid null argument");
                }
                o.append(" ");
                o.append(array[i]);
                ++i;
            }
            return DiagnosticCommandImpl.access$000(DiagnosticCommandImpl.this, o.toString());
        }
    }
    
    private static class OperationInfoComparator implements Comparator<MBeanOperationInfo>
    {
        @Override
        public int compare(MBeanOperationInfo mBeanOperationInfo, MBeanOperationInfo mBeanOperationInfo2) {
            return mBeanOperationInfo.getName().compareTo(mBeanOperationInfo2.getName());
        }
    }
}
